module Order where

import Polynomial as P
import MaxPlus as MP
import Precedence
import Status

data Order = PolyNat P.Algebra
           | PolyInt P.Algebra
           | PolyNegInt P.Algebra
           | LinearWPO P.Algebra Precedence Status
           | MaxPlusWPO MP.Algebra Precedence Status
           | LinearSPONat P.Algebra Precedence Status
           | LinearSPONegInt P.Algebra Precedence Status

instance Show Order where
  show (PolyNat a) =
    "linear polynomial interpretations on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a
  show (PolyInt a) =
    "linear polynomial interpretations on integers:\n" ++
    "\n" ++
    P.showAlgebra a
  show (PolyNegInt a) =
    "linear polynomial interpretations on negative integers:\n" ++
    "\n" ++
    P.showAlgebra a
  show (LinearWPO a p st) =
    "weighted path orer with linear polynomial on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (MaxPlusWPO a p st) =
    "weighted path orer with max/plus on natural numbers:\n" ++
    "\n" ++
    MP.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (LinearSPONat a p st) =
    "semantic path orer with linear polynomial on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (LinearSPONegInt a p st) =
    "semantic path orer with linear polynomial on negative integers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
