module Polynomial where

-- design choice: do not include domain-sensitive stuff!

import Data.List
import Data.Maybe

import Term hiding (variables)

-- f_0 + f_1 x_1 + ... + f_n x_n
type Coefficients = (Int, [Int])

-- f(x1,...,xn) = f_0 + f_1 x_1 + ... + f_n x_n
type Interpretation = (String,Coefficients)

type Algebra = [Interpretation]

showMonomial :: (Int, String) -> String
showMonomial (0, _) = "0"
showMonomial (1, x) = x
showMonomial (n, x) = show n ++ " " ++ x

-- print as polynomials
showCoefficients :: Coefficients -> String
showCoefficients (a0, as) = 
  case ms1 ++ ms2 of
    [] -> "0"
    ms -> intercalate " + " ms
  where
    ms1 = [ showMonomial (ai, "x" ++ show i) 
          | (i, ai) <- zip [1 :: Int ..] as, ai /= 0 ]
    ms2 = if a0 == 0 then [] else [show a0]

showInterpretation :: Interpretation -> String
showInterpretation (f, (a0,[])) =
  f ++ "_A = " ++ show a0
showInterpretation (f, p@(_,as)) =
  f ++ "_A(" ++ xs ++ ") = " ++ showCoefficients p
  where
    xs = intercalate "," [ "x" ++ show i | i <- [1 .. length as] ]

showAlgebra :: Algebra -> String
showAlgebra interpretations =
  unlines 
    [ "  " ++ showInterpretation interpretation 
    | interpretation <- interpretations ] ++ "\n"

-- comparison of terms under a fixed algebra

type Polynomial = [(String, Int)] -- linear polynomial
-- for example
-- [("x", 1), ("y" 3), ("", 2)] represents
-- x + 3y + 2

var :: String -> Polynomial
var x = [(x,1)]

num :: Int -> Polynomial
num n = [("",n)]

normalize :: Polynomial -> Polynomial
normalize []           = []
normalize ((x, m) : v) =
  (x, m + sum [ n | (y, n) <- v, x == y ]) : 
  normalize [ (y, n) | (y, n) <- v, x /= y ]

plus :: [Polynomial] -> Polynomial
plus vs = normalize (concat vs)

times :: Int -> Polynomial -> Polynomial
times k v = [ (x, k * n) | (x, n) <- v ]

coefficient :: String -> Polynomial -> Int
coefficient x v = fromMaybe 0 (lookup x v)

variables :: Polynomial -> [String]
variables v = [ x | (x, _) <- v ]

interpret :: Algebra -> Term -> Polynomial
interpret _ (V x) = var x 
interpret a (F f ts)
  | Just (a0, as) <- lookup f a = 
    plus (num a0 : [ times ai (interpret a ti) | (ai, ti) <- zip as ts ])
interpret _ _ = error "interpret"
